@@ -11,7 +11,8 @@ typedef enum {
 
 /* Representation used for VM thread handles. */
 struct MVMThreadBody {
-    /* The code object we will invoke to start the thread.. */
+    /* The code handle (MVMCode or MVMCFunction) we will invoke as the thread
+     * entrypoint. */
     MVMObject *invokee;
 
     /* The underlying OS thread handle. */
@@ -27,9 +28,14 @@ struct MVMThreadBody {
     AO_t stage;
 
     /* Thread's OS-level thread ID. */
-    MVMint64 thread_id;
+    MVMint64 native_thread_id;
 
-    /* Whether this thread should be automatically killed at VM exit. */
+    /* Copy of the VM-level thread ID (also stored in the ThreadContext while
+     * the thread is alive). */
+    MVMuint32 thread_id;
+
+    /* Non-zero if the thread should not block shutdown of the VM (those with
+     * zero in here will be joined when the main thread ends). */
     MVMint32 app_lifetime;
 };
 struct MVMThread {
