@@ -155,9 +155,6 @@ struct MVMThreadContext {
     MVMuint32        gc_work_size;
     MVMuint32        gc_work_count;
 
-    /* Per-thread fixed size allocator state. */
-    MVMFixedSizeAllocThread *thread_fsa;
-
     /************************************************************************
      * Interpreter state
      ************************************************************************/
@@ -278,7 +275,7 @@ struct MVMThreadContext {
     MVMStrHashTable native_callback_cache;
 
     /* Random number generator state. */
-    MVMuint64 rand_state[2];
+    MVMuint64 rand_state[4];
 
     /* Temporary big integers for when we need to upgrade operands in order
      * to perform an operation. */
