@@ -8,4 +8,4 @@ MVM_PUBLIC void MVM_telemetry_interval_annotate(uintptr_t subject, int intervalI
 MVM_PUBLIC void MVM_telemetry_interval_annotate_dynamic(uintptr_t subject, int intervalID, char *description);
 
 MVM_PUBLIC void MVM_telemetry_init(FILE *outfile);
-MVM_PUBLIC void MVM_telemetry_finish();
+MVM_PUBLIC void MVM_telemetry_finish(void);
